#!/usr/bin/python3

from random import randint, seed, shuffle
from string import ascii_lowercase as abc
seed(47)

MINN = [ None, 1, 2, 2, 2, 1 ]
MAXN = [ None, 250000, 100, 5000, 250000, 250000 ]
MIND = [ None, 0, 1, 1, 0, 0 ]
MAXD = [ None, 0, 100, 5000, 250000, 250000 ]
MAXF = [ None, 249999, 249999, 249999, 249999, 10**9 - 1 ]

last_saved = { x : 0 for x in range(1,6) }

def save(fname, T, D, plot):
    print('saving', fname)
    f = open(fname, 'w')
    print(T, file=f)
    print(len(plot), D, file=f)
    print(*plot, file=f)
    f.close()

def save_twice(batch, D, plot):
    global last_saved
    letter = abc[ last_saved[batch] ]
    save(f'{batch:02d}.{letter}.in', 1, D, plot)
    save(f'{batch+5:02d}.{letter}.in', 2, D, plot)
    last_saved[batch] += 1

def vyber_farby(pocet_farieb, max_farba, suvisle):
    if suvisle:
        zaciatok = randint(0, max_farba - pocet_farieb + 1)
        return list( range( zaciatok, zaciatok+pocet_farieb ) )
    else:
        hodnoty = set()
        while len(hodnoty) < pocet_farieb: hodnoty.add( randint(0, max_farba) )
        return list( hodnoty )

def gen_random(N, pocet_farieb, max_farba, suvisle=False):
    pocet_farieb = min( pocet_farieb, 2*N )
    farby = vyber_farby(pocet_farieb, max_farba, suvisle) 
    plot = [ farby[ randint(0, pocet_farieb-1) ] for n in range(N) ]
    return plot

def gen_runs(N, pocet_farieb, max_farba, avg_dlzka):
    farby = vyber_farby(pocet_farieb, max_farba, False) 
    curr = farby[ randint(0, pocet_farieb-1) ]
    plot = []
    for n in range(N):
        if randint(0, avg_dlzka-1) == 0:
            curr = farby[ randint(0, pocet_farieb-1) ]
        plot.append(curr)
    return plot

def gen_clusters(N, min_kusov, max_kusov, jump_length, max_farba):
    plot = []
    while len(plot) < N:
        kusov = randint(min_kusov, max_kusov)
        kusov = min(kusov, N - len(plot))
        where = randint(0, len(plot)+kusov-1)
        indices = set()
        while len(indices) < kusov:
            indices.add(where)
            where += randint(-jump_length, jump_length)
            where %= len(plot)+kusov
        new_plot = []
        new_color = randint(0, max_farba)
        for i in range(len(plot)+kusov):
            if i in indices:
                new_plot.append(new_color)
            else:
                new_plot.append(plot.pop())
        plot = new_plot
    return plot

def gen_counterexample(N, pocet_farieb, max_farba, suvisle=True):
    farby = vyber_farby(pocet_farieb+1, max_farba, suvisle) 
    special = farby.pop()
    shuffle( farby )
    plot = []
    while len(plot) < N: plot.extend( farby )
    kusov = (N // pocet_farieb) - 2
    plot = plot[:N-kusov]
    zac = randint(7, N - 2*kusov - 7 )
    for i in range(kusov):
        plot = plot[:zac+2*i] + [special] + plot[zac+2*i:]
    return plot

# príklady zo zadania

save( '00.sample.a.in', 2, 2, [7,3,7,7,9] )
save( '00.sample.b.in', 2, 1, [10,20,30,40,50] )
save( '00.sample.c.in', 1, 3, [7,7,7,7,7] )

# prvá sada: 1, 6 & malé čísla farieb, $d=0$ \\

save_twice(1, 0, gen_random(MAXN[1], 1, MAXF[1]) )
save_twice(1, 0, gen_random(MAXN[1], 2, MAXF[1]) )
save_twice(1, 0, gen_random(MAXN[1], 10, MAXF[1]) )
save_twice(1, 0, gen_random(MAXN[1], 1000, MAXF[1]) )
save_twice(1, 0, gen_runs(MAXN[1], 5, MAXF[1], 50000 ) )
save_twice(1, 0, gen_runs(MAXN[1], 500, MAXF[1], 500 ) )

# 2, 7 & malé čísla farieb, $2\leq n\leq 100$, $1\leq d\leq 100$ \\

save_twice(2, MAXD[2], [6, 7])
save_twice(2, MAXD[2], [7, 6])
save_twice(2, 1, [4, 4])
save_twice(2, 2, [4, 4])
save_twice(2, MAXD[2], gen_random(MAXN[2]-7, MAXN[2], MAXF[2]) )
save_twice(2, MAXD[2]//2, gen_random(MAXN[2], MAXN[2]//2, MAXF[2]) )
save_twice(2, 47, gen_random(MAXN[2], 10, MAXF[2], True) )
save_twice(2, 11, gen_random(MAXN[2], 3, MAXF[2]) )
save_twice(2, 5, gen_clusters(MAXN[2], 3, 6, 4, MAXF[2]) )
save_twice(2, 11, gen_clusters(MAXN[2], 2, 6, 6, MAXF[2]) )

# 3, 8 & malé čísla farieb, $2\leq n\leq 5\,000$, $1\leq d\leq 5\,000$ \\

save_twice(3, 5, gen_counterexample(50, 10, 99, True))
save_twice(3, 5, gen_counterexample(50, 10, 99, False))
save_twice(3, 123, gen_counterexample(MAXN[3], MAXN[3]//5, MAXF[3], True))
save_twice(3, 123, gen_counterexample(MAXN[3], MAXN[3]//5, MAXF[3], False))
save_twice(3, MAXD[3], gen_random(MAXN[3]-7, MAXN[3], MAXF[3]) )
save_twice(3, MAXD[3]//2, gen_random(MAXN[3], MAXN[3]//2, MAXF[3]) )
save_twice(3, MAXD[3]//10, gen_random(MAXN[3], 2, MAXF[3]) )
save_twice(3, MAXD[3]//10, gen_random(MAXN[3], 5, MAXF[3]) )
save_twice(3, MAXD[3]//10, gen_random(MAXN[3], 14, MAXF[3], True) )

# štvrtá sada: 4, 9 & malé čísla farieb, primálo času, $2\leq n$ \\

save_twice(4, 54321, gen_random( MAXN[4], MAXF[4], MAXF[4], True ) )
save_twice(4, 123456, gen_random( MAXN[4], MAXF[4]//10, MAXF[4], True ) )
save_twice(4, 76543, gen_random( MAXN[4], MAXF[4]//10, MAXF[4], False ) )
save_twice(4, 54321, gen_random( MAXN[4], 2, MAXF[4], False ) )
save_twice(4, 1234, gen_counterexample( MAXN[4], MAXN[4]//4, MAXF[4], True ) )
save_twice(4, 100000, gen_counterexample( MAXN[4], MAXN[4]//6, MAXF[4], False ) )
save_twice(4, 1000, gen_clusters( MAXN[4], 10, 30, 15, MAXF[4] ) )

# piata sada: 5, 10 &

save_twice(5, 1, [47])
save_twice(5, 1, [123456789]*MAXN[5] )
save_twice(5, 54321, gen_random( MAXN[5], MAXF[5], MAXF[5], True ) )
save_twice(5, 123456, gen_random( MAXN[5], MAXF[5]//10, MAXF[5], True ) )
save_twice(5, 76543, gen_random( MAXN[5], MAXF[5]//10, MAXF[5], False ) )
save_twice(5, 54321, gen_random( MAXN[5], 2, MAXF[5], False ) )
save_twice(5, 1234, gen_counterexample( MAXN[5], MAXN[5]//5, MAXF[5], True ) )
save_twice(5, 100000, gen_counterexample( MAXN[5], MAXN[5]//6, MAXF[5], False ) )
save_twice(5, 1000, gen_clusters( MAXN[5], 10, 30, 15, MAXF[5] ) )
save_twice(5, MAXD[5]//7, gen_random(MAXN[5], 3, MAXF[5]) )
save_twice(5, MAXD[5]//6, gen_random(MAXN[5], 6, MAXF[5]) )
save_twice(5, MAXD[5]//5, gen_random(MAXN[5], 12, MAXF[5], True) )
save_twice(5, MAXD[5], gen_random(MAXN[5]-7, MAXN[5], MAXF[5]) )
save_twice(5, MAXD[5] - 123 , gen_random(MAXN[5] - 7, 47, MAXF[5]) )
save_twice(5, MAXD[5] - 47, gen_random(MAXN[5], 3, MAXF[5]) )

